@@ -1,27 +1,30 @@
 """Table"""
 
-from sqlite3 import OperationalError
+from sqlite3 import Connection, OperationalError
 from typing import Any, Generator, Iterable, NamedTuple, Optional, Type
 
 import weakref
 
-from ._utils import AttrDict, WithCursor, check_iter, check_one, Ref
+
+from .utils import crunch
+from ._utils import check_iter, check_one
 from .column import BuilderColumn, Column
 from .errors import TableRemovedError, UnexpectedResultError
 from .locals import SQLITEPYTYPES
-from .query_builder import (Condition, build_update_data, combine_keyvals,
-                            extract_signature, extract_single_column,
-                            fetch_columns, format_paramable)
+from .query_builder import (Condition, extract_single_column,
+                            fetch_columns, build_select,
+                            build_insert, build_delete,
+                            build_update)
 from .signature import op
 from .typings import (Data, Orders, Queries, Query, TypicalNamedTuple,
-                      _MasterQuery)
+                      _MasterQuery, OnlyColumn)
+from .config import Config
 
 # Let's add a little bit of 'black' magic here.
 
 
 @classmethod
-def get_table(cls):
-    """Get table"""
+def get_table(cls):  # pylint: disable=missing-function-docstring
     return getattr(cls, '_table', None)
 
 
@@ -29,15 +32,17 @@ class Table:
     """Table. Make sure you remember how the table goes."""
 
     _ns: dict[str, Type[NamedTuple]] = {}
-    _parent = Ref()
 
     def __init__(self,
                  parent,  # type: ignore
                  table: str,
+                 db_config: Config | None = None,
                  __columns: Optional[Iterable[Column]] = None) -> None:
-        self._parent = parent
+        self._parent_repr = repr(parent)
+        self._sql: Connection = parent.sql
         self._deleted = False
         self._table = check_one(table)
+        self._config = db_config or Config(crunch=False)
         self._columns: Optional[list[Column]] = list(
             __columns) if __columns else None
         weakref.finalize(self, self._finalize)
@@ -57,9 +62,9 @@ class Table:
     def _fetch_columns(self):
         table = self._table
         try:
-            master = self._parent.table("sqlite_master")
-            tabl = master.select_one(
-                {"type": op == "table", "name": op == table})
+            query, data = build_select("sqlite_master",
+                                       {"type": op == "table", "name": op == table})
+            tabl = self._sql.execute(query, data).fetchone()
             if tabl is None:
                 raise ValueError("What the hell?")
             cols = fetch_columns(_MasterQuery(**tabl))
@@ -68,112 +73,51 @@ class Table:
         except Exception:  # pylint: disable=broad-except
             return 1
 
-    def _raw_exec(self, query: str, data: dict[str, Any]) -> WithCursor:
+    def _raw_exec(self, query: str, data: dict[str, Any]):
         """No thread safe :("""
-        cursor = self._parent.cursor()
+        cursor = self._sql.cursor()
         cursor.execute(query, data)
         return cursor
 
-    def _mksquery(self,
-                  filter_: Condition = None,
-                  limit: int = 0,
-                  offset: int = 0,
-                  order: Optional[Orders] = None):
-        cond, data = extract_signature(filter_)
-        query = f"select * from {self._table}{' '+cond if cond else ''}"
-        if limit:
-            query += f" limit {limit}"
-        if offset:
-            query += f" offset {offset}"
-        if order:
-            query += " order by"
-            for ord_, order_by in order.items():
-                query += f" {ord_} {order_by},"
-            query = query[:-1]
-        # print(query, data)
-        return query, data
-
-    def _mkuquery(self,
-                  new_data: Data,
-                  filter_: Condition = None,
-                  limit: int = 0,
-                  order: Optional[Orders] = None):
-        cond, data = extract_signature(filter_)
-        new_str, updated = build_update_data(new_data)
-        query = f"update {self._table} set {new_str} {cond}"
-        if limit:
-            query += f" limit {limit}"
-        if order:
-            query += " order by"
-            for ord_, order_by in order.items():
-                query += f" {ord_} {order_by},"
-            query = query[:-1]
-        # print(query, data | _combine_keyvals(updated, new_data))
-        return query, data | combine_keyvals(updated, new_data)
-
-    def _mkdquery(self,
-                  filter_: Condition = None,
-                  limit: int = 0,
-                  order: Optional[Orders] = None):
-        cond, data = extract_signature(filter_)
-        query = f"delete from {self._table} {cond}"
-        if limit:
-            query += f" limit {limit}"
-        if order:
-            query += " order by"
-            for ord_, order_by in order.items():
-                query += f" {ord_} {order_by}"
-            query = query[:-1]
-        # print(query, data)
-        return query, data
-
-    def _mkiquery(self, data: Data):
-        converged = format_paramable(data)
-        query = f"insert into {self._table} ({', '.join(val for val in converged)}) \
-values ({', '.join(val for _,val in converged.items())})"
-        # print(query, data)
-        return query, data
 
     def _control(self):
         if self._deleted:
             raise TableRemovedError(f"{self._table} is already removed")
-        if self._parent.closed:
-            raise ConnectionError("Connection to database was closed.\
- A operation was held but database is already closed.")
 
     def force_nodelete(self):
         """Force "undelete" table. Used if table was mistakenly assigned as
         deleted."""
         self._deleted = True
 
-    def delete(self, filter_: Condition = None, limit: int = 0, order: Optional[Orders] = None):
+    def delete(self, condition: Condition = None, limit: int = 0, order: Optional[Orders] = None):
         """Delete row or rows
 
         Args:
-            filter_ (Condition, optional): Condition to determine deletion
-            See `Signature` class about conditional stuff. Defaults to None.
+            condition (Condition, optional): Condition to determine deletion
+                See `Signature` class about conditional stuff. Defaults to None.
             limit (int, optional): Limit deletion by integer. Defaults to 0.
             order (Optional[Orders], optional): Order of deletion. Defaults to None.
 
         Returns:
             int: Rows affected
         """
-        query, data = self._mkdquery(filter_, limit, order)
+        query, data = build_delete(
+            self._table, condition, limit, order)  # type: ignore
         self._control()
-        cursor = self._parent.sql.execute(query, data)
+        cursor = self._sql.execute(query, data)
         rcount = cursor.rowcount
-        self._parent.sql.commit()
+        self._sql.commit()
         return rcount
 
-    def delete_one(self, filter_: Condition = None, order: Optional[Orders] = None):
+    def delete_one(self, condition: Condition = None, order: Optional[Orders] = None):
         """Delete a row
 
         Args:
-            filter_ (Condition, optional): Conditional to determine deletion.
+            condition (Condition, optional): Conditional to determine deletion.
             Defaults to None.
             order (Optional[Orders], optional): Order of deletion. Defaults to None.
         """
-        return self.delete(filter_, 1, order)
+        return self.delete(condition, 1, order)
 
     def insert(self, data: Data):
         """Insert data to current table
@@ -184,30 +128,30 @@ values ({', '.join(val for _,val in converged.items())})"
         Returns:
             int: Last rowid
         """
-        query, _ = self._mkiquery(data)
+        query, _ = build_insert(self._table, data)  # type: ignore
         self._control()
-        cursor = self._parent.sql.execute(query, data)
+        cursor = self._sql.execute(query, data)
         rlastrowid = cursor.lastrowid
-        self._parent.sql.commit()
+        self._sql.commit()
         return rlastrowid
 
-    def insert_multiple(self, datas: Iterable[Data]):
+    def insert_multiple(self, datas: list[Data]):
         """Insert multiple values
 
         Args:
             datas (Iterable[Data]): Data to be inserted.
         """
         self._control()
-        query, _ = self._mkiquery(datas[0])
-        self._parent.sql.executemany(query, datas)
-        self._parent.sql.commit()
+        query, _ = build_insert(self._table, datas[0])  # type: ignore
+        self._sql.executemany(query, datas)
+        self._sql.commit()
 
-    def insert_many(self, datas: Iterable[Data]):
+    def insert_many(self, datas: list[Data]):
         """Alias to `insert_multiple`"""
         return self.insert_multiple(datas)
 
     def update(self,
-               filter_: Condition | None = None,
+               condition: Condition | None = None,
                data: Data | None = None,
                limit: int = 0,
                order: Optional[Orders] = None):
@@ -215,8 +159,8 @@ values ({', '.join(val for _,val in converged.items())})"
 
         Args:
             data (Data): New data to update
-            filter_ (Condition, optional): Condition dictionary
-            See `Signature` about how filter_ works. Defaults to None.
+            condition (Condition, optional): Condition dictionary. 
+                See `Signature` about how condition works. Defaults to None.
             limit (int, optional): Limit updates. Defaults to 0.
             order (Optional[Orders], optional): Order of change. Defaults to None.
 
@@ -225,29 +169,32 @@ values ({', '.join(val for _,val in converged.items())})"
         """
         if data is None:
             raise ValueError("data parameter must not be None")
-        query, data = self._mkuquery(data, filter_, limit, order)
+        query, data = build_update(
+            self._table, data, condition, limit, order)  # type: ignore
         self._control()
-        cursor = self._parent.sql.execute(query, data)
+        cursor = self._sql.execute(query, data)
         rcount = cursor.rowcount
-        self._parent.sql.commit()
+        self._sql.commit()
         return rcount
 
     def update_one(self,
-                   filter_: Condition | None = None,
+                   condition: Condition | None = None,
                    new_data: Data | None = None,
                    order: Orders | None = None) -> int:
         """Update 1 data only"""
-        return self.update(filter_, new_data, 1, order)
+        return self.update(condition, new_data, 1, order)
 
-    def select(self,
-               filter_: Condition = None,
+    def select(self, # pylint: disable=too-many-arguments
+               condition: Condition = None,
+               only: tuple[str, ...] | None = None,
                limit: int = 0,
                offset: int = 0,
                order: Optional[Orders] = None) -> Queries:
         """Select data in current table. Bare .select() returns all data.
 
         Args:
-            filter_ (Condition, optional): Conditions to used. Defaults to None.
+            condition (Condition, optional): Conditions to used. Defaults to None.
+            only: (OnlyColumn, optional): Select what you want. Default to None.
             limit (int, optional): Limit of select. Defaults to 0.
             offset (int, optional): Offset. Defaults to 0
             order (Optional[Orders], optional): Selection order. Defaults to None.
@@ -256,18 +203,28 @@ values ({', '.join(val for _,val in converged.items())})"
             Queries: Selected data
         """
         self._control()
-        query, data = self._mksquery(filter_, limit, offset, order)
-        with self._parent.sql:
-            return self._parent.sql.execute(query, data).fetchall()
+        query, data = build_select(self._table,
+                                   condition,
+                                   only,
+                                   limit,
+                                   offset,
+                                   order)  # type: ignore
+        with self._sql:
+            data = self._sql.execute(query, data).fetchall()
+            if self._config['crunch']:
+                return crunch(data)
+            return data
 
     def paginate_select(self,
-                        filter_: Condition = None,
+                        condition: Condition = None,
+                        only: OnlyColumn = None,
                         length: int = 10,
                         order: Optional[Orders] = None) -> Generator[Queries, None, None]:
         """Paginate select
 
         Args:
-            filter_ (Condition, optional): Confitions to use. Defaults to None.
+            condition (Condition, optional): Confitions to use. Defaults to None.
+            only: (OnlyColumn, optional): Select what you want. Default to None.
             length (int, optional): Pagination length. Defaults to 10.
             order (Optional[Orders], optional): Order. Defaults to None.
 
@@ -277,12 +234,20 @@ values ({', '.join(val for _,val in converged.items())})"
         self._control()
         start = 0
         while True:
-            query, data = self._mksquery(filter_, length, start, order)
-            with self._parent.sql:
-                fetched: list[AttrDict] = self._parent.sql.execute(
+            query, data = build_select(self._table,
+                                       condition,
+                                       only,
+                                       length,
+                                       start,
+                                       order)  # type: ignore
+            crunched = self._config["crunch"]
+            with self._sql:
+                fetched = self._sql.execute(
                     query, data).fetchmany(length)
                 if len(fetched) == 0:
                     return
+                if crunched:
+                    fetched = crunch(fetched)
                 if len(fetched) != length:
                     yield fetched
                     return
@@ -290,29 +255,32 @@ values ({', '.join(val for _,val in converged.items())})"
                 start += length
 
     def select_one(self,
-                   filter_: Condition = None,
+                   condition: Condition = None,
+                   only: OnlyColumn = None,
                    order: Optional[Orders] = None) -> Query | None:
         """Select one data
 
         Args:
-            filter_ (Condition, optional): Condition to use. Defaults to None.
+            condition (Condition, optional): Condition to use. Defaults to None.
+            only: (OnlyColumn, optional): Select what you want. Default to None.
             order (Optional[Orders], optional): Order of selection. Defaults to None.
 
         Returns:
             Query: Selected data
         """
         self._control()
-        query, data = self._mksquery(filter_, 1, 0, order)
-        with self._parent.sql:
-            return self._parent.sql.execute(query, data).fetchone()
+        query, data = build_select(
+            self._table, condition, only, 1, 0, order)  # type: ignore
+        with self._sql:
+            return self._sql.execute(query, data).fetchone()
 
-    def exists(self, filter_: Condition = None):
+    def exists(self, condition: Condition = None):
         """Check if data is exists or not.
 
         Args:
-            filter_ (Condition, optional): Condition to use. Defaults to None.
+            condition (Condition, optional): Condition to use. Defaults to None.
         """
-        data = self.select_one(filter_)
+        data = self.select_one(condition)
         if data is None:
             return False
         return True
@@ -345,11 +313,11 @@ values ({', '.join(val for _,val in converged.items())})"
         self._ns[self._table] = namedtupled
         return namedtupled
 
-    @property
     def columns(self):
         """Table columns"""
         if self._columns is None:
             raise AttributeError("columns is undefined.")
+
         return tuple(self._columns)
 
     @property
@@ -364,7 +332,7 @@ values ({', '.join(val for _,val in converged.items())})"
 
     def add_column(self, column: Column | BuilderColumn):
         """Add column to table"""
-        sql = self._parent.sql
+        sql = self._sql
         column = column.to_column() if isinstance(column, BuilderColumn) else column
         if column.primary or column.unique:
             raise OperationalError(
@@ -384,10 +352,10 @@ constraint is enabled.")
         """Rename existing column to new column"""
         check_iter((old_column, new_column))
         query = f"alter table {self._table} rename column {old_column} to {new_column}"
-        self._parent.sql.execute(query)
+        self._sql.execute(query)
 
     def __repr__(self) -> str:
-        return f"<Table({self._table}) -> {self._parent}>"
+        return f"<Table({self._table}) -> {self._parent_repr}>"
 
 
-__all__ = ['Table', 'get_table']
+__all__ = ['Table']
